# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse
from scale_gp.types.evaluations import (
    TestCaseResult,
    TestCaseResultWithViews,
    TestCaseResultBatchResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTestCaseResults:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        test_case_result = client.evaluations.test_case_results.create(
            evaluation_id="evaluation_id",
            application_spec_id="application_spec_id",
            evaluation_dataset_version_num="evaluation_dataset_version_num",
            test_case_evaluation_data={},
            test_case_id="test_case_id",
        )
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        test_case_result = client.evaluations.test_case_results.create(
            evaluation_id="evaluation_id",
            application_spec_id="application_spec_id",
            evaluation_dataset_version_num="evaluation_dataset_version_num",
            test_case_evaluation_data={},
            test_case_id="test_case_id",
            account_id="account_id",
            annotated_by_user_id="annotated_by_user_id",
            audit_comment="audit_comment",
            audit_required=True,
            audit_status="UNAUDITED",
            result={},
            time_spent_labeling_s=0,
        )
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.evaluations.test_case_results.with_raw_response.create(
            evaluation_id="evaluation_id",
            application_spec_id="application_spec_id",
            evaluation_dataset_version_num="evaluation_dataset_version_num",
            test_case_evaluation_data={},
            test_case_id="test_case_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case_result = response.parse()
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.evaluations.test_case_results.with_streaming_response.create(
            evaluation_id="evaluation_id",
            application_spec_id="application_spec_id",
            evaluation_dataset_version_num="evaluation_dataset_version_num",
            test_case_evaluation_data={},
            test_case_id="test_case_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case_result = response.parse()
            assert_matches_type(TestCaseResult, test_case_result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_id` but received ''"):
            client.evaluations.test_case_results.with_raw_response.create(
                evaluation_id="",
                application_spec_id="application_spec_id",
                evaluation_dataset_version_num="evaluation_dataset_version_num",
                test_case_evaluation_data={},
                test_case_id="test_case_id",
            )

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        test_case_result = client.evaluations.test_case_results.retrieve(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        )
        assert_matches_type(TestCaseResultWithViews, test_case_result, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: SGPClient) -> None:
        test_case_result = client.evaluations.test_case_results.retrieve(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
            view=["AnnotationResults", "Metrics", "Task"],
        )
        assert_matches_type(TestCaseResultWithViews, test_case_result, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.evaluations.test_case_results.with_raw_response.retrieve(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case_result = response.parse()
        assert_matches_type(TestCaseResultWithViews, test_case_result, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.evaluations.test_case_results.with_streaming_response.retrieve(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case_result = response.parse()
            assert_matches_type(TestCaseResultWithViews, test_case_result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_id` but received ''"):
            client.evaluations.test_case_results.with_raw_response.retrieve(
                test_case_result_id="test_case_result_id",
                evaluation_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_result_id` but received ''"):
            client.evaluations.test_case_results.with_raw_response.retrieve(
                test_case_result_id="",
                evaluation_id="evaluation_id",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        test_case_result = client.evaluations.test_case_results.update(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        )
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        test_case_result = client.evaluations.test_case_results.update(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
            annotated_by_user_id="annotated_by_user_id",
            application_spec_id="application_spec_id",
            audit_comment="audit_comment",
            audit_required=True,
            audit_status="UNAUDITED",
            evaluation_dataset_version_num="evaluation_dataset_version_num",
            result={},
            test_case_evaluation_data={},
            test_case_id="test_case_id",
            time_spent_labeling_s=0,
        )
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.evaluations.test_case_results.with_raw_response.update(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case_result = response.parse()
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.evaluations.test_case_results.with_streaming_response.update(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case_result = response.parse()
            assert_matches_type(TestCaseResult, test_case_result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_id` but received ''"):
            client.evaluations.test_case_results.with_raw_response.update(
                test_case_result_id="test_case_result_id",
                evaluation_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_result_id` but received ''"):
            client.evaluations.test_case_results.with_raw_response.update(
                test_case_result_id="",
                evaluation_id="evaluation_id",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        test_case_result = client.evaluations.test_case_results.list(
            evaluation_id="evaluation_id",
        )
        assert_matches_type(SyncPageResponse[TestCaseResultWithViews], test_case_result, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        test_case_result = client.evaluations.test_case_results.list(
            evaluation_id="evaluation_id",
            account_id="account_id",
            include_archived=True,
            limit=1,
            page=1,
            view=["AnnotationResults", "Metrics", "Task"],
        )
        assert_matches_type(SyncPageResponse[TestCaseResultWithViews], test_case_result, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.evaluations.test_case_results.with_raw_response.list(
            evaluation_id="evaluation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case_result = response.parse()
        assert_matches_type(SyncPageResponse[TestCaseResultWithViews], test_case_result, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.evaluations.test_case_results.with_streaming_response.list(
            evaluation_id="evaluation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case_result = response.parse()
            assert_matches_type(SyncPageResponse[TestCaseResultWithViews], test_case_result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_id` but received ''"):
            client.evaluations.test_case_results.with_raw_response.list(
                evaluation_id="",
            )

    @parametrize
    def test_method_batch(self, client: SGPClient) -> None:
        test_case_result = client.evaluations.test_case_results.batch(
            evaluation_id="evaluation_id",
            items=[
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
            ],
        )
        assert_matches_type(TestCaseResultBatchResponse, test_case_result, path=["response"])

    @parametrize
    def test_raw_response_batch(self, client: SGPClient) -> None:
        response = client.evaluations.test_case_results.with_raw_response.batch(
            evaluation_id="evaluation_id",
            items=[
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case_result = response.parse()
        assert_matches_type(TestCaseResultBatchResponse, test_case_result, path=["response"])

    @parametrize
    def test_streaming_response_batch(self, client: SGPClient) -> None:
        with client.evaluations.test_case_results.with_streaming_response.batch(
            evaluation_id="evaluation_id",
            items=[
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case_result = response.parse()
            assert_matches_type(TestCaseResultBatchResponse, test_case_result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_batch(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_id` but received ''"):
            client.evaluations.test_case_results.with_raw_response.batch(
                evaluation_id="",
                items=[
                    {
                        "application_spec_id": "application_spec_id",
                        "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                        "test_case_evaluation_data": {},
                        "test_case_id": "test_case_id",
                    },
                    {
                        "application_spec_id": "application_spec_id",
                        "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                        "test_case_evaluation_data": {},
                        "test_case_id": "test_case_id",
                    },
                    {
                        "application_spec_id": "application_spec_id",
                        "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                        "test_case_evaluation_data": {},
                        "test_case_id": "test_case_id",
                    },
                ],
            )


class TestAsyncTestCaseResults:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        test_case_result = await async_client.evaluations.test_case_results.create(
            evaluation_id="evaluation_id",
            application_spec_id="application_spec_id",
            evaluation_dataset_version_num="evaluation_dataset_version_num",
            test_case_evaluation_data={},
            test_case_id="test_case_id",
        )
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        test_case_result = await async_client.evaluations.test_case_results.create(
            evaluation_id="evaluation_id",
            application_spec_id="application_spec_id",
            evaluation_dataset_version_num="evaluation_dataset_version_num",
            test_case_evaluation_data={},
            test_case_id="test_case_id",
            account_id="account_id",
            annotated_by_user_id="annotated_by_user_id",
            audit_comment="audit_comment",
            audit_required=True,
            audit_status="UNAUDITED",
            result={},
            time_spent_labeling_s=0,
        )
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluations.test_case_results.with_raw_response.create(
            evaluation_id="evaluation_id",
            application_spec_id="application_spec_id",
            evaluation_dataset_version_num="evaluation_dataset_version_num",
            test_case_evaluation_data={},
            test_case_id="test_case_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case_result = await response.parse()
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluations.test_case_results.with_streaming_response.create(
            evaluation_id="evaluation_id",
            application_spec_id="application_spec_id",
            evaluation_dataset_version_num="evaluation_dataset_version_num",
            test_case_evaluation_data={},
            test_case_id="test_case_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case_result = await response.parse()
            assert_matches_type(TestCaseResult, test_case_result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_id` but received ''"):
            await async_client.evaluations.test_case_results.with_raw_response.create(
                evaluation_id="",
                application_spec_id="application_spec_id",
                evaluation_dataset_version_num="evaluation_dataset_version_num",
                test_case_evaluation_data={},
                test_case_id="test_case_id",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        test_case_result = await async_client.evaluations.test_case_results.retrieve(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        )
        assert_matches_type(TestCaseResultWithViews, test_case_result, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSGPClient) -> None:
        test_case_result = await async_client.evaluations.test_case_results.retrieve(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
            view=["AnnotationResults", "Metrics", "Task"],
        )
        assert_matches_type(TestCaseResultWithViews, test_case_result, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluations.test_case_results.with_raw_response.retrieve(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case_result = await response.parse()
        assert_matches_type(TestCaseResultWithViews, test_case_result, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluations.test_case_results.with_streaming_response.retrieve(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case_result = await response.parse()
            assert_matches_type(TestCaseResultWithViews, test_case_result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_id` but received ''"):
            await async_client.evaluations.test_case_results.with_raw_response.retrieve(
                test_case_result_id="test_case_result_id",
                evaluation_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_result_id` but received ''"):
            await async_client.evaluations.test_case_results.with_raw_response.retrieve(
                test_case_result_id="",
                evaluation_id="evaluation_id",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        test_case_result = await async_client.evaluations.test_case_results.update(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        )
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        test_case_result = await async_client.evaluations.test_case_results.update(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
            annotated_by_user_id="annotated_by_user_id",
            application_spec_id="application_spec_id",
            audit_comment="audit_comment",
            audit_required=True,
            audit_status="UNAUDITED",
            evaluation_dataset_version_num="evaluation_dataset_version_num",
            result={},
            test_case_evaluation_data={},
            test_case_id="test_case_id",
            time_spent_labeling_s=0,
        )
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluations.test_case_results.with_raw_response.update(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case_result = await response.parse()
        assert_matches_type(TestCaseResult, test_case_result, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluations.test_case_results.with_streaming_response.update(
            test_case_result_id="test_case_result_id",
            evaluation_id="evaluation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case_result = await response.parse()
            assert_matches_type(TestCaseResult, test_case_result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_id` but received ''"):
            await async_client.evaluations.test_case_results.with_raw_response.update(
                test_case_result_id="test_case_result_id",
                evaluation_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `test_case_result_id` but received ''"):
            await async_client.evaluations.test_case_results.with_raw_response.update(
                test_case_result_id="",
                evaluation_id="evaluation_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        test_case_result = await async_client.evaluations.test_case_results.list(
            evaluation_id="evaluation_id",
        )
        assert_matches_type(AsyncPageResponse[TestCaseResultWithViews], test_case_result, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        test_case_result = await async_client.evaluations.test_case_results.list(
            evaluation_id="evaluation_id",
            account_id="account_id",
            include_archived=True,
            limit=1,
            page=1,
            view=["AnnotationResults", "Metrics", "Task"],
        )
        assert_matches_type(AsyncPageResponse[TestCaseResultWithViews], test_case_result, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluations.test_case_results.with_raw_response.list(
            evaluation_id="evaluation_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case_result = await response.parse()
        assert_matches_type(AsyncPageResponse[TestCaseResultWithViews], test_case_result, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluations.test_case_results.with_streaming_response.list(
            evaluation_id="evaluation_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case_result = await response.parse()
            assert_matches_type(AsyncPageResponse[TestCaseResultWithViews], test_case_result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_id` but received ''"):
            await async_client.evaluations.test_case_results.with_raw_response.list(
                evaluation_id="",
            )

    @parametrize
    async def test_method_batch(self, async_client: AsyncSGPClient) -> None:
        test_case_result = await async_client.evaluations.test_case_results.batch(
            evaluation_id="evaluation_id",
            items=[
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
            ],
        )
        assert_matches_type(TestCaseResultBatchResponse, test_case_result, path=["response"])

    @parametrize
    async def test_raw_response_batch(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluations.test_case_results.with_raw_response.batch(
            evaluation_id="evaluation_id",
            items=[
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        test_case_result = await response.parse()
        assert_matches_type(TestCaseResultBatchResponse, test_case_result, path=["response"])

    @parametrize
    async def test_streaming_response_batch(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluations.test_case_results.with_streaming_response.batch(
            evaluation_id="evaluation_id",
            items=[
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
                {
                    "application_spec_id": "application_spec_id",
                    "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                    "test_case_evaluation_data": {},
                    "test_case_id": "test_case_id",
                },
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            test_case_result = await response.parse()
            assert_matches_type(TestCaseResultBatchResponse, test_case_result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_batch(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_id` but received ''"):
            await async_client.evaluations.test_case_results.with_raw_response.batch(
                evaluation_id="",
                items=[
                    {
                        "application_spec_id": "application_spec_id",
                        "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                        "test_case_evaluation_data": {},
                        "test_case_id": "test_case_id",
                    },
                    {
                        "application_spec_id": "application_spec_id",
                        "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                        "test_case_evaluation_data": {},
                        "test_case_id": "test_case_id",
                    },
                    {
                        "application_spec_id": "application_spec_id",
                        "evaluation_dataset_version_num": "evaluation_dataset_version_num",
                        "test_case_evaluation_data": {},
                        "test_case_id": "test_case_id",
                    },
                ],
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = ["EvaluationDatasetUpdateParams", "PatchEvaluationDatasetRequest", "RestoreRequest"]


class PatchEvaluationDatasetRequest(TypedDict, total=False):
    name: Required[str]
    """The name of the dataset"""

    restore: Literal[False]
    """Set to true to restore the entity from the database."""


class RestoreRequest(TypedDict, total=False):
    restore: Required[Literal[True]]
    """Set to true to restore the entity from the database."""


EvaluationDatasetUpdateParams: TypeAlias = Union[PatchEvaluationDatasetRequest, RestoreRequest]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AccountCreateParams"]


class AccountCreateParams(TypedDict, total=False):
    account_name: Required[str]
    """The name of the account. This will be displayed in the Scale GenAI Platform UI."""



### evaluation_example_script.py ###
# This script demonstrates how to perform an evaluation on a specified application variant
# 1. Create at least one question
# 2. Create a question set using the desired questions
# 3. Create an evaluation config using the question set and a desired evaluation type
# 4. Create an evaluation using the evaluation config, and the desired application variant
# 5. Retrieve the evaluation results

# What is an evaluation?
# Evaluate an application variant against a rubric (question set) to determine how well the application variant performs.
# Depending on the evaluation type, either use a LLM, human, or mixed (LLM Benchmark) to compare the response of an application variant to an expected response
# The results of the evaluation can be used to help generate a report card for the application variant, benchmark performance, and compare variants.

# Equivalent to https://egp.dashboard.scale.com/evaluation/runs/new?new_evaluation_app_id=&new_evaluation_variant_id=&account_id=<account_id>

import os
import time

from scale_gp import SGPClient
from scale_gp.types.application_edge_param import ApplicationEdgeParam
from scale_gp.types.application_node_param import ApplicationNodeParam
from scale_gp.types.model_template_create_params import (
    VendorConfiguration,
    VendorConfigurationBundleConfig,
    VendorConfigurationEndpointConfig,
)
from scale_gp.types.application_configuration_param import ApplicationConfigurationParam

account_id = os.environ.get("SGP_ACCOUNT_ID", None)
api_key = os.environ.get("SGP_API_KEY", None)

assert (
    account_id is not None
), "You need to set the SGP_ACCOUNT_ID - you can find it at https://egp.dashboard.scale.com/admin/accounts"
assert api_key is not None, "You need to provide your API key - see https://egp.dashboard.scale.com/admin/api-key"

# Create an SGP Client
client = SGPClient(api_key=api_key)

### SETUP ###
bundle_config = VendorConfigurationBundleConfig(image="gemini-pro", registry="aws-registry", tag="latest")
endpoint_config = VendorConfigurationEndpointConfig(
    max_workers=3,
)

vendor_configuration = VendorConfiguration(
    bundle_config=bundle_config,
    endpoint_config=endpoint_config,
)
model_template = client.model_templates.create(
    account_id=account_id,
    endpoint_type="SYNC",
    model_type="COMPLETION",
    name="Gemini-Pro Template",
    vendor_configuration=vendor_configuration,
)

model_instance = client.models.create(
    account_id=account_id,
    model_type="COMPLETION",
    name="gemini-pro",
    model_vendor="GOOGLE",
    model_template_id=model_template.id,
)
model_deployment = client.models.deployments.create(
    model_instance_id=model_instance.id, name="Gemini-Pro Deployment", account_id=account_id
)

input_node_id = "input_node"
completion_node_id = "completion_node"
output_node_id = "output_node"
application_spec = client.application_specs.create(
    account_id=account_id,
    description="Test application spec",
    name="test-application-spec" + str(time.time()),
)
edges = [
    ApplicationEdgeParam(
        from_node=input_node_id,
        to_node=completion_node_id,
        from_field="input-text",
        to_field="prompt",
    ),
    ApplicationEdgeParam(
        from_node=completion_node_id,
        to_node=output_node_id,
        from_field="completion",
        to_field="output-text",
    ),
]

nodes = [
    ApplicationNodeParam(
        id=input_node_id,
        application_node_schema_id="text_input_schema",
        configuration={"input-text": {"type": "TEXT", "value": "Completion prompt"}},
    ),
    ApplicationNodeParam(
        id=completion_node_id,
        application_node_schema_id="completion_model_schema",
        configuration={
            "completion_model_id": {
                "type": "TEXT",
                "value": model_deployment.id,  # define the deployed completion model we want to use inside the application
            }
        },
    ),
    ApplicationNodeParam(
        id=output_node_id,
        application_node_schema_id="text_output_schema",
        configuration={"completion": {"type": "TEXT", "value": "completion result"}},
    ),
]
configuration = ApplicationConfigurationParam(edges=edges, nodes=nodes)
application_variant = client.application_variants.create(
    account_id=account_id,
    application_spec_id=application_spec.id,
    name="test-application-variant",
    version="V0",
    description="Test application variant",
    configuration=configuration,
)

DATASET = [
    {
        "input": "What is the capital of France?",
        "expected_output": "The capital of France is Paris.",
        "expected_extra_info": {
            "info": "Paris, located in the northern part of France along the Seine River, is not only the country’s capital but also its largest city. Known for its art, fashion, gastronomy, and culture, Paris has a significant influence both in France and globally.",
            "schema_type": "STRING",
        },
    },
    {
        "input": "What color is an apple?",
        "expected_output": "An apple is typically red when ripe, though green and yellow varieties also exist depending on the species and maturity.",
        "expected_extra_info": {
            "info": "Apples come in various colors including red, green, and yellow. Red apples are one of the most common, with varieties like Red Delicious being very popular. Green apples, like Granny Smith, are tart and used often in baking. Yellow apples such as Golden Delicious are sweet and softer.",
            "schema_type": "STRING",
        },
    },
    {
        "input": "Who is the first president of the USA?",
        "expected_output": "The first president of the USA is George Washington.",
        "expected_extra_info": {
            "info": "George Washington served as the first president of the United States from 1789 to 1797. He is a pivotal figure in American history, recognized for his leadership during the Revolutionary War and setting many precedents for the national government.",
            "schema_type": "STRING",
        },
    },
]

evaluation_dataset = client.evaluation_datasets.create(
    account_id=account_id,
    name="evaluation_dataset",
    schema_type="GENERATION",
    type="manual",
)
uploaded_test_cases = client.evaluation_datasets.test_cases.batch(
    evaluation_dataset_id=evaluation_dataset.id,
    items=[{"account_id": account_id, "schema_type": "GENERATION", "test_case_data": item} for item in DATASET],
)

manual_dataset_version = client.evaluation_datasets.evaluation_dataset_versions.create(
    evaluation_dataset_id=evaluation_dataset.id
)

published_dataset_response = client.evaluation_datasets.publish(
    evaluation_dataset_id=evaluation_dataset.id,
)
### SETUP END ###

# 1. create at least one question
question = client.questions.create(
    account_id=account_id,
    prompt="Why is the sky blue? Explain the Rayleigh Scattering Effect.",
    title="Rayleigh Scattering Effect",
    type="free_text",
    dropdown=True,
    multi=False,
    required=True,
)

# 2. create a question set using the desired questions
question_set = client.question_sets.create(
    account_id=account_id,
    name="Rayleigh Scattering Effect Question Set",
    question_ids=[question.id],
)

# 3. create an evaluation config
evaluation_config = client.evaluation_configs.create(
    account_id=account_id,
    evaluation_type="llm_auto",  # ["llm_auto", "human", "llm_benchmark"]
    question_set_id=question_set.id,
)

# 4. create an evaluation using the question set, and application spec/variant created in the application_process_example.py script
# use the manual evaluation dataset created in the eval_dataset_example_script.py script
evaluation = client.evaluations.create(
    account_id=account_id,
    application_spec_id=application_spec.id,
    description="Test-evaluation",
    name="test-evaluation",
    application_variant_id=application_variant.id,
    type="builder",
    evaluation_config_id=evaluation_config.id,
    evaluation_dataset_id=evaluation_dataset.id,
)

# 5. Retrieve the evaluation results
while True:
    print("Polling for evaluation results")
    evaluation_results = client.evaluations.retrieve(evaluation_id=evaluation.id, view="TestCaseResults")
    if evaluation_results.status != "PENDING":
        break
    time.sleep(5)

print("Evaluation Results:")
print(evaluation_results)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CustomChunkingStrategyConfigParam"]


class CustomChunkingStrategyConfigParam(TypedDict, total=False):
    endpoint: Required[str]
    """Endpoint path to call for custom chunking"""

    strategy: Required[Literal["custom"]]

    params: object
    """Parameters that will be appended to the body of the request for the chunk."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncTopLevelArray, AsyncTopLevelArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.fine_tuning_job_event import FineTuningJobEvent

__all__ = ["EventsResource", "AsyncEventsResource"]


class EventsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return EventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return EventsResourceWithStreamingResponse(self)

    def list(
        self,
        fine_tuning_job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncTopLevelArray[FineTuningJobEvent]:
        """
        Get fine tuning job events

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not fine_tuning_job_id:
            raise ValueError(f"Expected a non-empty value for `fine_tuning_job_id` but received {fine_tuning_job_id!r}")
        return self._get_api_list(
            f"/v4/fine-tuning-jobs/{fine_tuning_job_id}/events",
            page=SyncTopLevelArray[FineTuningJobEvent],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FineTuningJobEvent,
        )


class AsyncEventsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncEventsResourceWithStreamingResponse(self)

    def list(
        self,
        fine_tuning_job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[FineTuningJobEvent, AsyncTopLevelArray[FineTuningJobEvent]]:
        """
        Get fine tuning job events

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not fine_tuning_job_id:
            raise ValueError(f"Expected a non-empty value for `fine_tuning_job_id` but received {fine_tuning_job_id!r}")
        return self._get_api_list(
            f"/v4/fine-tuning-jobs/{fine_tuning_job_id}/events",
            page=AsyncTopLevelArray[FineTuningJobEvent],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FineTuningJobEvent,
        )


class EventsResourceWithRawResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.list = to_raw_response_wrapper(
            events.list,
        )


class AsyncEventsResourceWithRawResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.list = async_to_raw_response_wrapper(
            events.list,
        )


class EventsResourceWithStreamingResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.list = to_streamed_response_wrapper(
            events.list,
        )


class AsyncEventsResourceWithStreamingResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.list = async_to_streamed_response_wrapper(
            events.list,
        )



from typing import Any, Dict, Union, Optional

from scale_gp import BaseModel


class SummarizationTestCaseSchema(BaseModel):
    document: str
    expected_summary: Optional[str]
    other_inputs: Optional[Union[str, float, Dict[str, Any]]] = None
    other_expected: Optional[Union[str, float, Dict[str, Any]]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from ...types import fine_tuning_job_list_params, fine_tuning_job_create_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from ...types.fine_tuning_job import FineTuningJob
from ...types.shared.generic_delete_response import GenericDeleteResponse

__all__ = ["FineTuningJobsResource", "AsyncFineTuningJobsResource"]


class FineTuningJobsResource(SyncAPIResource):
    @cached_property
    def events(self) -> EventsResource:
        return EventsResource(self._client)

    @cached_property
    def with_raw_response(self) -> FineTuningJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return FineTuningJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FineTuningJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return FineTuningJobsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        training_dataset_id: str,
        base_model_id: str | NotGiven = NOT_GIVEN,
        fine_tuned_model_id: str | NotGiven = NOT_GIVEN,
        validation_dataset_id: str | NotGiven = NOT_GIVEN,
        vendor_configuration: fine_tuning_job_create_params.VendorConfiguration | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FineTuningJob:
        """
        ### Description

        Model fine tuning jobs.

        ### Details

        TODO

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/fine-tuning-jobs",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "training_dataset_id": training_dataset_id,
                    "base_model_id": base_model_id,
                    "fine_tuned_model_id": fine_tuned_model_id,
                    "validation_dataset_id": validation_dataset_id,
                    "vendor_configuration": vendor_configuration,
                },
                fine_tuning_job_create_params.FineTuningJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningJob,
        )

    def retrieve(
        self,
        fine_tuning_job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FineTuningJob:
        """
        ### Description

        Gets the details of a fine tuning job

        ### Details

        This API can be used to get information about a single fine tuning job by ID. To
        use this API, pass in the `id` that was returned from your Create Fine Tuning
        Job API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not fine_tuning_job_id:
            raise ValueError(f"Expected a non-empty value for `fine_tuning_job_id` but received {fine_tuning_job_id!r}")
        return self._get(
            f"/v4/fine-tuning-jobs/{fine_tuning_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningJob,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[FineTuningJob]:
        """
        ### Description

        Lists all fine tuning jobs accessible to the user.

        ### Details

        This API can be used to list fine tuning jobs. If a user has access to multiple
        accounts, all fine tuning jobs from all accounts the user is associated with
        will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/fine-tuning-jobs",
            page=SyncPageResponse[FineTuningJob],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    fine_tuning_job_list_params.FineTuningJobListParams,
                ),
            ),
            model=FineTuningJob,
        )

    def delete(
        self,
        fine_tuning_job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a fine tuning job

        ### Details

        This API can be used to delete a fine tuning job by ID. To use this API, pass in
        the `id` that was returned from your Create Fine Tuning Job API call as a path
        parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not fine_tuning_job_id:
            raise ValueError(f"Expected a non-empty value for `fine_tuning_job_id` but received {fine_tuning_job_id!r}")
        return self._delete(
            f"/v4/fine-tuning-jobs/{fine_tuning_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class AsyncFineTuningJobsResource(AsyncAPIResource):
    @cached_property
    def events(self) -> AsyncEventsResource:
        return AsyncEventsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFineTuningJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFineTuningJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFineTuningJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncFineTuningJobsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        training_dataset_id: str,
        base_model_id: str | NotGiven = NOT_GIVEN,
        fine_tuned_model_id: str | NotGiven = NOT_GIVEN,
        validation_dataset_id: str | NotGiven = NOT_GIVEN,
        vendor_configuration: fine_tuning_job_create_params.VendorConfiguration | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FineTuningJob:
        """
        ### Description

        Model fine tuning jobs.

        ### Details

        TODO

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/fine-tuning-jobs",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "training_dataset_id": training_dataset_id,
                    "base_model_id": base_model_id,
                    "fine_tuned_model_id": fine_tuned_model_id,
                    "validation_dataset_id": validation_dataset_id,
                    "vendor_configuration": vendor_configuration,
                },
                fine_tuning_job_create_params.FineTuningJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningJob,
        )

    async def retrieve(
        self,
        fine_tuning_job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FineTuningJob:
        """
        ### Description

        Gets the details of a fine tuning job

        ### Details

        This API can be used to get information about a single fine tuning job by ID. To
        use this API, pass in the `id` that was returned from your Create Fine Tuning
        Job API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not fine_tuning_job_id:
            raise ValueError(f"Expected a non-empty value for `fine_tuning_job_id` but received {fine_tuning_job_id!r}")
        return await self._get(
            f"/v4/fine-tuning-jobs/{fine_tuning_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningJob,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[FineTuningJob, AsyncPageResponse[FineTuningJob]]:
        """
        ### Description

        Lists all fine tuning jobs accessible to the user.

        ### Details

        This API can be used to list fine tuning jobs. If a user has access to multiple
        accounts, all fine tuning jobs from all accounts the user is associated with
        will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/fine-tuning-jobs",
            page=AsyncPageResponse[FineTuningJob],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    fine_tuning_job_list_params.FineTuningJobListParams,
                ),
            ),
            model=FineTuningJob,
        )

    async def delete(
        self,
        fine_tuning_job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a fine tuning job

        ### Details

        This API can be used to delete a fine tuning job by ID. To use this API, pass in
        the `id` that was returned from your Create Fine Tuning Job API call as a path
        parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not fine_tuning_job_id:
            raise ValueError(f"Expected a non-empty value for `fine_tuning_job_id` but received {fine_tuning_job_id!r}")
        return await self._delete(
            f"/v4/fine-tuning-jobs/{fine_tuning_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class FineTuningJobsResourceWithRawResponse:
    def __init__(self, fine_tuning_jobs: FineTuningJobsResource) -> None:
        self._fine_tuning_jobs = fine_tuning_jobs

        self.create = to_raw_response_wrapper(
            fine_tuning_jobs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            fine_tuning_jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            fine_tuning_jobs.list,
        )
        self.delete = to_raw_response_wrapper(
            fine_tuning_jobs.delete,
        )

    @cached_property
    def events(self) -> EventsResourceWithRawResponse:
        return EventsResourceWithRawResponse(self._fine_tuning_jobs.events)


class AsyncFineTuningJobsResourceWithRawResponse:
    def __init__(self, fine_tuning_jobs: AsyncFineTuningJobsResource) -> None:
        self._fine_tuning_jobs = fine_tuning_jobs

        self.create = async_to_raw_response_wrapper(
            fine_tuning_jobs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            fine_tuning_jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            fine_tuning_jobs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            fine_tuning_jobs.delete,
        )

    @cached_property
    def events(self) -> AsyncEventsResourceWithRawResponse:
        return AsyncEventsResourceWithRawResponse(self._fine_tuning_jobs.events)


class FineTuningJobsResourceWithStreamingResponse:
    def __init__(self, fine_tuning_jobs: FineTuningJobsResource) -> None:
        self._fine_tuning_jobs = fine_tuning_jobs

        self.create = to_streamed_response_wrapper(
            fine_tuning_jobs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            fine_tuning_jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            fine_tuning_jobs.list,
        )
        self.delete = to_streamed_response_wrapper(
            fine_tuning_jobs.delete,
        )

    @cached_property
    def events(self) -> EventsResourceWithStreamingResponse:
        return EventsResourceWithStreamingResponse(self._fine_tuning_jobs.events)


class AsyncFineTuningJobsResourceWithStreamingResponse:
    def __init__(self, fine_tuning_jobs: AsyncFineTuningJobsResource) -> None:
        self._fine_tuning_jobs = fine_tuning_jobs

        self.create = async_to_streamed_response_wrapper(
            fine_tuning_jobs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            fine_tuning_jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            fine_tuning_jobs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            fine_tuning_jobs.delete,
        )

    @cached_property
    def events(self) -> AsyncEventsResourceWithStreamingResponse:
        return AsyncEventsResourceWithStreamingResponse(self._fine_tuning_jobs.events)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestApplicationThreads:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="prisma failure")
    @parametrize
    def test_method_process(self, client: SGPClient) -> None:
        application_thread = client.application_threads.process(
            thread_id="thread_id",
            application_variant_id="application_variant_id",
            inputs={},
        )
        assert_matches_type(object, application_thread, path=["response"])

    @pytest.mark.skip(reason="prisma failure")
    @parametrize
    def test_method_process_with_all_params(self, client: SGPClient) -> None:
        application_thread = client.application_threads.process(
            thread_id="thread_id",
            application_variant_id="application_variant_id",
            inputs={},
            history=[
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
            ],
            overrides={
                "foo": {
                    "artifact_ids_filter": ["string", "string", "string"],
                    "artifact_name_regex": ["string", "string", "string"],
                    "type": "knowledge_base_schema",
                }
            },
            stream=True,
        )
        assert_matches_type(object, application_thread, path=["response"])

    @pytest.mark.skip(reason="prisma failure")
    @parametrize
    def test_raw_response_process(self, client: SGPClient) -> None:
        response = client.application_threads.with_raw_response.process(
            thread_id="thread_id",
            application_variant_id="application_variant_id",
            inputs={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_thread = response.parse()
        assert_matches_type(object, application_thread, path=["response"])

    @pytest.mark.skip(reason="prisma failure")
    @parametrize
    def test_streaming_response_process(self, client: SGPClient) -> None:
        with client.application_threads.with_streaming_response.process(
            thread_id="thread_id",
            application_variant_id="application_variant_id",
            inputs={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_thread = response.parse()
            assert_matches_type(object, application_thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="prisma failure")
    @parametrize
    def test_path_params_process(self, client: SGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_variant_id` but received ''"
        ):
            client.application_threads.with_raw_response.process(
                thread_id="thread_id",
                application_variant_id="",
                inputs={},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            client.application_threads.with_raw_response.process(
                thread_id="",
                application_variant_id="application_variant_id",
                inputs={},
            )


class TestAsyncApplicationThreads:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="prisma failure")
    @parametrize
    async def test_method_process(self, async_client: AsyncSGPClient) -> None:
        application_thread = await async_client.application_threads.process(
            thread_id="thread_id",
            application_variant_id="application_variant_id",
            inputs={},
        )
        assert_matches_type(object, application_thread, path=["response"])

    @pytest.mark.skip(reason="prisma failure")
    @parametrize
    async def test_method_process_with_all_params(self, async_client: AsyncSGPClient) -> None:
        application_thread = await async_client.application_threads.process(
            thread_id="thread_id",
            application_variant_id="application_variant_id",
            inputs={},
            history=[
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
            ],
            overrides={
                "foo": {
                    "artifact_ids_filter": ["string", "string", "string"],
                    "artifact_name_regex": ["string", "string", "string"],
                    "type": "knowledge_base_schema",
                }
            },
            stream=True,
        )
        assert_matches_type(object, application_thread, path=["response"])

    @pytest.mark.skip(reason="prisma failure")
    @parametrize
    async def test_raw_response_process(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_threads.with_raw_response.process(
            thread_id="thread_id",
            application_variant_id="application_variant_id",
            inputs={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_thread = await response.parse()
        assert_matches_type(object, application_thread, path=["response"])

    @pytest.mark.skip(reason="prisma failure")
    @parametrize
    async def test_streaming_response_process(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_threads.with_streaming_response.process(
            thread_id="thread_id",
            application_variant_id="application_variant_id",
            inputs={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_thread = await response.parse()
            assert_matches_type(object, application_thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="prisma failure")
    @parametrize
    async def test_path_params_process(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_variant_id` but received ''"
        ):
            await async_client.application_threads.with_raw_response.process(
                thread_id="thread_id",
                application_variant_id="",
                inputs={},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            await async_client.application_threads.with_raw_response.process(
                thread_id="",
                application_variant_id="application_variant_id",
                inputs={},
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, overload
from typing_extensions import Literal

import httpx

from ..types import chat_completion_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._streaming import Stream, AsyncStream
from .._base_client import make_request_options
from ..types.chat_completions_response import ChatCompletionsResponse
from ..types.shared.completion_response import CompletionResponse

__all__ = ["ChatCompletionsResource", "AsyncChatCompletionsResource"]


class ChatCompletionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChatCompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ChatCompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatCompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ChatCompletionsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        messages: Iterable[chat_completion_create_params.Message],
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-4o",
            "gpt-4o-2024-08-06",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "gemini-pro",
            "gemini-1.5-pro-001",
            "gemini-1.5-pro-preview-0409",
            "gemini-1.5-pro-preview-0514",
            "llama-2-7b-chat",
            "llama-2-13b-chat",
            "llama-2-70b-chat",
            "llama-3-8b-instruct",
            "llama-3-70b-instruct",
            "llama-3-1-8b-instruct",
            "llama-3-1-70b-instruct",
            "Meta-Llama-3-8B-Instruct-RMU",
            "Meta-Llama-3-8B-Instruct-RR",
            "Meta-Llama-3-8B-Instruct-DERTA",
            "Meta-Llama-3-8B-Instruct-LAT",
            "mixtral-8x7b-instruct",
            "mixtral-8x22b-instruct",
            "claude-3-opus-20240229",
            "claude-3-sonnet-20240229",
            "claude-3-haiku-20240307",
            "claude-3-5-sonnet-20240620",
            "mistral-large-latest",
            "phi-3-mini-4k-instruct",
            "phi-3-cat-merged",
            "zephyr-cat-merged",
            "dolphin-2.9-llama3-8b",
            "dolphin-2.9-llama3-70b",
            "llama3-1-405b-instruct-v1",
        ],
        account_id: str | NotGiven = NOT_GIVEN,
        chat_template: str | NotGiven = NOT_GIVEN,
        instructions: str | NotGiven = NOT_GIVEN,
        memory_strategy: chat_completion_create_params.MemoryStrategy | NotGiven = NOT_GIVEN,
        model_parameters: chat_completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        stream: Literal[False] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCompletionsResponse:
        """
        ### Description

        Given a list of messages representing a conversation history, runs LLM inference
        to produce the next message.

        ### Details

        Like [completions](https://scale-egp.readme.io/docs/completions-1),
        [chat completions](https://scale-egp.readme.io/docs/chat-completions-intro)
        involve an LLM's response to input. However, chat completions take a
        conversation history as input, instead of a single prompt, which enables the LLM
        to create responses that take past context into account.

        ### Messages

        The primary input to the LLM is a list of messages represented by the `messages`
        array, which forms the conversation. The `messages` array must contain at least
        one `message` object. Each `message` object is attributed to a specific entity
        through its `role`. The available roles are:

        - `user`: Represents the human querying the model. - `assistant`: Represents the
          model responding to user. - `system`: Represents a non-user entity that
          provides information to guide the behavior of the assistant.

        When the `role` of a `message` is set to `user`, `assistant`, or `system`, the
        `message` must also contain a `content` field which is a string representing the
        actual text of the message itself. Semantically, when the `role` is `user`,
        `content` contains the user's query. When the `role` is `assistant`, `content`
        is the model's response to the user. When the `role` is `system`, `content`
        represents the instruction for the assistant.

        ### Instructions

        You may provide instructions to the assistant by supplying by supplying
        `instructions` in the HTTP request body or by specifying a `message` with `role`
        set to `system` in the `messages` array. By convention, the system message
        should be the first message in the array. Do **not** specify both an instruction
        and a system message in the `messages` array.

        Args:
          messages: The list of messages in the conversation.

              Expand each message type to see how it works and when to use it. Most
              conversations should begin with a single `user` message.

          model: The ID of the model to use for chat completions. We only support the models
              listed here so far.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          chat_template: Currently only supported for LLM-Engine models. A Jinja template string that
              defines how the chat completion API formats the string prompt. For Llama models,
              the template must take in at most a `messages` object, `bos_token` string, and
              `eos_token` string. The `messages` object is a list of dictionaries, each with
              keys `role` and `content`. For Mixtral models, the template must take in at most
              a `messages` object and `eos_token` string. The `messages` object looks
              identical to the Llama model's `messages` object, but the template can assume
              the `role` key takes on the values `user` or `assistant`, or `system` for the
              first message. The chat template either needs to handle this system message
              (which gets set via the `instructions` field or by the messages), or the
              `instructions` field must be set to `null` and the `messages` object must not
              contain any system messages.See the default chat template present in the Llama
              and Mixtral tokenizers for examples.

          instructions: The initial instructions to provide to the chat completion model.

              Use this to guide the model to act in more specific ways. For example, if you
              have specific rules you want to restrict the model to follow you can specify
              them here.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing more
              specific instructions here before trying more expensive techniques such as
              swapping in other models or finetuning the underlying LLM.

          memory_strategy: The memory strategy to use for the agent. A memory strategy is a way to prevent
              the underlying LLM's context limit from being exceeded. Each memory strategy
              uses a different technique to condense the input message list into a smaller
              payload for the underlying LLM.

              We only support the Last K memory strategy right now, but will be adding new
              strategies soon.

          model_parameters: Configuration parameters for the chat completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        messages: Iterable[chat_completion_create_params.Message],
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-4o",
            "gpt-4o-2024-08-06",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "gemini-pro",
            "gemini-1.5-pro-001",
            "gemini-1.5-pro-preview-0409",
            "gemini-1.5-pro-preview-0514",
            "llama-2-7b-chat",
            "llama-2-13b-chat",
            "llama-2-70b-chat",
            "llama-3-8b-instruct",
            "llama-3-70b-instruct",
            "llama-3-1-8b-instruct",
            "llama-3-1-70b-instruct",
            "Meta-Llama-3-8B-Instruct-RMU",
            "Meta-Llama-3-8B-Instruct-RR",
            "Meta-Llama-3-8B-Instruct-DERTA",
            "Meta-Llama-3-8B-Instruct-LAT",
            "mixtral-8x7b-instruct",
            "mixtral-8x22b-instruct",
            "claude-3-opus-20240229",
            "claude-3-sonnet-20240229",
            "claude-3-haiku-20240307",
            "claude-3-5-sonnet-20240620",
            "mistral-large-latest",
            "phi-3-mini-4k-instruct",
            "phi-3-cat-merged",
            "zephyr-cat-merged",
            "dolphin-2.9-llama3-8b",
            "dolphin-2.9-llama3-70b",
            "llama3-1-405b-instruct-v1",
        ],
        stream: Literal[True],
        account_id: str | NotGiven = NOT_GIVEN,
        chat_template: str | NotGiven = NOT_GIVEN,
        instructions: str | NotGiven = NOT_GIVEN,
        memory_strategy: chat_completion_create_params.MemoryStrategy | NotGiven = NOT_GIVEN,
        model_parameters: chat_completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Stream[CompletionResponse]:
        """
        ### Description

        Given a list of messages representing a conversation history, runs LLM inference
        to produce the next message.

        ### Details

        Like [completions](https://scale-egp.readme.io/docs/completions-1),
        [chat completions](https://scale-egp.readme.io/docs/chat-completions-intro)
        involve an LLM's response to input. However, chat completions take a
        conversation history as input, instead of a single prompt, which enables the LLM
        to create responses that take past context into account.

        ### Messages

        The primary input to the LLM is a list of messages represented by the `messages`
        array, which forms the conversation. The `messages` array must contain at least
        one `message` object. Each `message` object is attributed to a specific entity
        through its `role`. The available roles are:

        - `user`: Represents the human querying the model. - `assistant`: Represents the
          model responding to user. - `system`: Represents a non-user entity that
          provides information to guide the behavior of the assistant.

        When the `role` of a `message` is set to `user`, `assistant`, or `system`, the
        `message` must also contain a `content` field which is a string representing the
        actual text of the message itself. Semantically, when the `role` is `user`,
        `content` contains the user's query. When the `role` is `assistant`, `content`
        is the model's response to the user. When the `role` is `system`, `content`
        represents the instruction for the assistant.

        ### Instructions

        You may provide instructions to the assistant by supplying by supplying
        `instructions` in the HTTP request body or by specifying a `message` with `role`
        set to `system` in the `messages` array. By convention, the system message
        should be the first message in the array. Do **not** specify both an instruction
        and a system message in the `messages` array.

        Args:
          messages: The list of messages in the conversation.

              Expand each message type to see how it works and when to use it. Most
              conversations should begin with a single `user` message.

          model: The ID of the model to use for chat completions. We only support the models
              listed here so far.

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          chat_template: Currently only supported for LLM-Engine models. A Jinja template string that
              defines how the chat completion API formats the string prompt. For Llama models,
              the template must take in at most a `messages` object, `bos_token` string, and
              `eos_token` string. The `messages` object is a list of dictionaries, each with
              keys `role` and `content`. For Mixtral models, the template must take in at most
              a `messages` object and `eos_token` string. The `messages` object looks
              identical to the Llama model's `messages` object, but the template can assume
              the `role` key takes on the values `user` or `assistant`, or `system` for the
              first message. The chat template either needs to handle this system message
              (which gets set via the `instructions` field or by the messages), or the
              `instructions` field must be set to `null` and the `messages` object must not
              contain any system messages.See the default chat template present in the Llama
              and Mixtral tokenizers for examples.

          instructions: The initial instructions to provide to the chat completion model.

              Use this to guide the model to act in more specific ways. For example, if you
              have specific rules you want to restrict the model to follow you can specify
              them here.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing more
              specific instructions here before trying more expensive techniques such as
              swapping in other models or finetuning the underlying LLM.

          memory_strategy: The memory strategy to use for the agent. A memory strategy is a way to prevent
              the underlying LLM's context limit from being exceeded. Each memory strategy
              uses a different technique to condense the input message list into a smaller
              payload for the underlying LLM.

              We only support the Last K memory strategy right now, but will be adding new
              strategies soon.

          model_parameters: Configuration parameters for the chat completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        messages: Iterable[chat_completion_create_params.Message],
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-4o",
            "gpt-4o-2024-08-06",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "gemini-pro",
            "gemini-1.5-pro-001",
            "gemini-1.5-pro-preview-0409",
            "gemini-1.5-pro-preview-0514",
            "llama-2-7b-chat",
            "llama-2-13b-chat",
            "llama-2-70b-chat",
            "llama-3-8b-instruct",
            "llama-3-70b-instruct",
            "llama-3-1-8b-instruct",
            "llama-3-1-70b-instruct",
            "Meta-Llama-3-8B-Instruct-RMU",
            "Meta-Llama-3-8B-Instruct-RR",
            "Meta-Llama-3-8B-Instruct-DERTA",
            "Meta-Llama-3-8B-Instruct-LAT",
            "mixtral-8x7b-instruct",
            "mixtral-8x22b-instruct",
            "claude-3-opus-20240229",
            "claude-3-sonnet-20240229",
            "claude-3-haiku-20240307",
            "claude-3-5-sonnet-20240620",
            "mistral-large-latest",
            "phi-3-mini-4k-instruct",
            "phi-3-cat-merged",
            "zephyr-cat-merged",
            "dolphin-2.9-llama3-8b",
            "dolphin-2.9-llama3-70b",
            "llama3-1-405b-instruct-v1",
        ],
        stream: bool,
        account_id: str | NotGiven = NOT_GIVEN,
        chat_template: str | NotGiven = NOT_GIVEN,
        instructions: str | NotGiven = NOT_GIVEN,
        memory_strategy: chat_completion_create_params.MemoryStrategy | NotGiven = NOT_GIVEN,
        model_parameters: chat_completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCompletionsResponse | Stream[CompletionResponse]:
        """
        ### Description

        Given a list of messages representing a conversation history, runs LLM inference
        to produce the next message.

        ### Details

        Like [completions](https://scale-egp.readme.io/docs/completions-1),
        [chat completions](https://scale-egp.readme.io/docs/chat-completions-intro)
        involve an LLM's response to input. However, chat completions take a
        conversation history as input, instead of a single prompt, which enables the LLM
        to create responses that take past context into account.

        ### Messages

        The primary input to the LLM is a list of messages represented by the `messages`
        array, which forms the conversation. The `messages` array must contain at least
        one `message` object. Each `message` object is attributed to a specific entity
        through its `role`. The available roles are:

        - `user`: Represents the human querying the model. - `assistant`: Represents the
          model responding to user. - `system`: Represents a non-user entity that
          provides information to guide the behavior of the assistant.

        When the `role` of a `message` is set to `user`, `assistant`, or `system`, the
        `message` must also contain a `content` field which is a string representing the
        actual text of the message itself. Semantically, when the `role` is `user`,
        `content` contains the user's query. When the `role` is `assistant`, `content`
        is the model's response to the user. When the `role` is `system`, `content`
        represents the instruction for the assistant.

        ### Instructions

        You may provide instructions to the assistant by supplying by supplying
        `instructions` in the HTTP request body or by specifying a `message` with `role`
        set to `system` in the `messages` array. By convention, the system message
        should be the first message in the array. Do **not** specify both an instruction
        and a system message in the `messages` array.

        Args:
          messages: The list of messages in the conversation.

              Expand each message type to see how it works and when to use it. Most
              conversations should begin with a single `user` message.

          model: The ID of the model to use for chat completions. We only support the models
              listed here so far.

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          chat_template: Currently only supported for LLM-Engine models. A Jinja template string that
              defines how the chat completion API formats the string prompt. For Llama models,
              the template must take in at most a `messages` object, `bos_token` string, and
              `eos_token` string. The `messages` object is a list of dictionaries, each with
              keys `role` and `content`. For Mixtral models, the template must take in at most
              a `messages` object and `eos_token` string. The `messages` object looks
              identical to the Llama model's `messages` object, but the template can assume
              the `role` key takes on the values `user` or `assistant`, or `system` for the
              first message. The chat template either needs to handle this system message
              (which gets set via the `instructions` field or by the messages), or the
              `instructions` field must be set to `null` and the `messages` object must not
              contain any system messages.See the default chat template present in the Llama
              and Mixtral tokenizers for examples.

          instructions: The initial instructions to provide to the chat completion model.

              Use this to guide the model to act in more specific ways. For example, if you
              have specific rules you want to restrict the model to follow you can specify
              them here.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing more
              specific instructions here before trying more expensive techniques such as
              swapping in other models or finetuning the underlying LLM.

          memory_strategy: The memory strategy to use for the agent. A memory strategy is a way to prevent
              the underlying LLM's context limit from being exceeded. Each memory strategy
              uses a different technique to condense the input message list into a smaller
              payload for the underlying LLM.

              We only support the Last K memory strategy right now, but will be adding new
              strategies soon.

          model_parameters: Configuration parameters for the chat completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["messages", "model"], ["messages", "model", "stream"])
    def create(
        self,
        *,
        messages: Iterable[chat_completion_create_params.Message],
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-4o",
            "gpt-4o-2024-08-06",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "gemini-pro",
            "gemini-1.5-pro-001",
            "gemini-1.5-pro-preview-0409",
            "gemini-1.5-pro-preview-0514",
            "llama-2-7b-chat",
            "llama-2-13b-chat",
            "llama-2-70b-chat",
            "llama-3-8b-instruct",
            "llama-3-70b-instruct",
            "llama-3-1-8b-instruct",
            "llama-3-1-70b-instruct",
            "Meta-Llama-3-8B-Instruct-RMU",
            "Meta-Llama-3-8B-Instruct-RR",
            "Meta-Llama-3-8B-Instruct-DERTA",
            "Meta-Llama-3-8B-Instruct-LAT",
            "mixtral-8x7b-instruct",
            "mixtral-8x22b-instruct",
            "claude-3-opus-20240229",
            "claude-3-sonnet-20240229",
            "claude-3-haiku-20240307",
            "claude-3-5-sonnet-20240620",
            "mistral-large-latest",
            "phi-3-mini-4k-instruct",
            "phi-3-cat-merged",
            "zephyr-cat-merged",
            "dolphin-2.9-llama3-8b",
            "dolphin-2.9-llama3-70b",
            "llama3-1-405b-instruct-v1",
        ],
        account_id: str | NotGiven = NOT_GIVEN,
        chat_template: str | NotGiven = NOT_GIVEN,
        instructions: str | NotGiven = NOT_GIVEN,
        memory_strategy: chat_completion_create_params.MemoryStrategy | NotGiven = NOT_GIVEN,
        model_parameters: chat_completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        stream: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCompletionsResponse | Stream[CompletionResponse]:
        return self._post(
            "/v4/chat-completions",
            body=maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "account_id": account_id,
                    "chat_template": chat_template,
                    "instructions": instructions,
                    "memory_strategy": memory_strategy,
                    "model_parameters": model_parameters,
                    "stream": stream,
                },
                chat_completion_create_params.ChatCompletionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCompletionsResponse,
            stream=stream or False,
            stream_cls=Stream[CompletionResponse],
        )


class AsyncChatCompletionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChatCompletionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChatCompletionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatCompletionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncChatCompletionsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        messages: Iterable[chat_completion_create_params.Message],
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-4o",
            "gpt-4o-2024-08-06",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "gemini-pro",
            "gemini-1.5-pro-001",
            "gemini-1.5-pro-preview-0409",
            "gemini-1.5-pro-preview-0514",
            "llama-2-7b-chat",
            "llama-2-13b-chat",
            "llama-2-70b-chat",
            "llama-3-8b-instruct",
            "llama-3-70b-instruct",
            "llama-3-1-8b-instruct",
            "llama-3-1-70b-instruct",
            "Meta-Llama-3-8B-Instruct-RMU",
            "Meta-Llama-3-8B-Instruct-RR",
            "Meta-Llama-3-8B-Instruct-DERTA",
            "Meta-Llama-3-8B-Instruct-LAT",
            "mixtral-8x7b-instruct",
            "mixtral-8x22b-instruct",
            "claude-3-opus-20240229",
            "claude-3-sonnet-20240229",
            "claude-3-haiku-20240307",
            "claude-3-5-sonnet-20240620",
            "mistral-large-latest",
            "phi-3-mini-4k-instruct",
            "phi-3-cat-merged",
            "zephyr-cat-merged",
            "dolphin-2.9-llama3-8b",
            "dolphin-2.9-llama3-70b",
            "llama3-1-405b-instruct-v1",
        ],
        account_id: str | NotGiven = NOT_GIVEN,
        chat_template: str | NotGiven = NOT_GIVEN,
        instructions: str | NotGiven = NOT_GIVEN,
        memory_strategy: chat_completion_create_params.MemoryStrategy | NotGiven = NOT_GIVEN,
        model_parameters: chat_completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        stream: Literal[False] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCompletionsResponse:
        """
        ### Description

        Given a list of messages representing a conversation history, runs LLM inference
        to produce the next message.

        ### Details

        Like [completions](https://scale-egp.readme.io/docs/completions-1),
        [chat completions](https://scale-egp.readme.io/docs/chat-completions-intro)
        involve an LLM's response to input. However, chat completions take a
        conversation history as input, instead of a single prompt, which enables the LLM
        to create responses that take past context into account.

        ### Messages

        The primary input to the LLM is a list of messages represented by the `messages`
        array, which forms the conversation. The `messages` array must contain at least
        one `message` object. Each `message` object is attributed to a specific entity
        through its `role`. The available roles are:

        - `user`: Represents the human querying the model. - `assistant`: Represents the
          model responding to user. - `system`: Represents a non-user entity that
          provides information to guide the behavior of the assistant.

        When the `role` of a `message` is set to `user`, `assistant`, or `system`, the
        `message` must also contain a `content` field which is a string representing the
        actual text of the message itself. Semantically, when the `role` is `user`,
        `content` contains the user's query. When the `role` is `assistant`, `content`
        is the model's response to the user. When the `role` is `system`, `content`
        represents the instruction for the assistant.

        ### Instructions

        You may provide instructions to the assistant by supplying by supplying
        `instructions` in the HTTP request body or by specifying a `message` with `role`
        set to `system` in the `messages` array. By convention, the system message
        should be the first message in the array. Do **not** specify both an instruction
        and a system message in the `messages` array.

        Args:
          messages: The list of messages in the conversation.

              Expand each message type to see how it works and when to use it. Most
              conversations should begin with a single `user` message.

          model: The ID of the model to use for chat completions. We only support the models
              listed here so far.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          chat_template: Currently only supported for LLM-Engine models. A Jinja template string that
              defines how the chat completion API formats the string prompt. For Llama models,
              the template must take in at most a `messages` object, `bos_token` string, and
              `eos_token` string. The `messages` object is a list of dictionaries, each with
              keys `role` and `content`. For Mixtral models, the template must take in at most
              a `messages` object and `eos_token` string. The `messages` object looks
              identical to the Llama model's `messages` object, but the template can assume
              the `role` key takes on the values `user` or `assistant`, or `system` for the
              first message. The chat template either needs to handle this system message
              (which gets set via the `instructions` field or by the messages), or the
              `instructions` field must be set to `null` and the `messages` object must not
              contain any system messages.See the default chat template present in the Llama
              and Mixtral tokenizers for examples.

          instructions: The initial instructions to provide to the chat completion model.

              Use this to guide the model to act in more specific ways. For example, if you
              have specific rules you want to restrict the model to follow you can specify
              them here.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing more
              specific instructions here before trying more expensive techniques such as
              swapping in other models or finetuning the underlying LLM.

          memory_strategy: The memory strategy to use for the agent. A memory strategy is a way to prevent
              the underlying LLM's context limit from being exceeded. Each memory strategy
              uses a different technique to condense the input message list into a smaller
              payload for the underlying LLM.

              We only support the Last K memory strategy right now, but will be adding new
              strategies soon.

          model_parameters: Configuration parameters for the chat completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        messages: Iterable[chat_completion_create_params.Message],
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-4o",
            "gpt-4o-2024-08-06",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "gemini-pro",
            "gemini-1.5-pro-001",
            "gemini-1.5-pro-preview-0409",
            "gemini-1.5-pro-preview-0514",
            "llama-2-7b-chat",
            "llama-2-13b-chat",
            "llama-2-70b-chat",
            "llama-3-8b-instruct",
            "llama-3-70b-instruct",
            "llama-3-1-8b-instruct",
            "llama-3-1-70b-instruct",
            "Meta-Llama-3-8B-Instruct-RMU",
            "Meta-Llama-3-8B-Instruct-RR",
            "Meta-Llama-3-8B-Instruct-DERTA",
            "Meta-Llama-3-8B-Instruct-LAT",
            "mixtral-8x7b-instruct",
            "mixtral-8x22b-instruct",
            "claude-3-opus-20240229",
            "claude-3-sonnet-20240229",
            "claude-3-haiku-20240307",
            "claude-3-5-sonnet-20240620",
            "mistral-large-latest",
            "phi-3-mini-4k-instruct",
            "phi-3-cat-merged",
            "zephyr-cat-merged",
            "dolphin-2.9-llama3-8b",
            "dolphin-2.9-llama3-70b",
            "llama3-1-405b-instruct-v1",
        ],
        stream: Literal[True],
        account_id: str | NotGiven = NOT_GIVEN,
        chat_template: str | NotGiven = NOT_GIVEN,
        instructions: str | NotGiven = NOT_GIVEN,
        memory_strategy: chat_completion_create_params.MemoryStrategy | NotGiven = NOT_GIVEN,
        model_parameters: chat_completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncStream[CompletionResponse]:
        """
        ### Description

        Given a list of messages representing a conversation history, runs LLM inference
        to produce the next message.

        ### Details

        Like [completions](https://scale-egp.readme.io/docs/completions-1),
        [chat completions](https://scale-egp.readme.io/docs/chat-completions-intro)
        involve an LLM's response to input. However, chat completions take a
        conversation history as input, instead of a single prompt, which enables the LLM
        to create responses that take past context into account.

        ### Messages

        The primary input to the LLM is a list of messages represented by the `messages`
        array, which forms the conversation. The `messages` array must contain at least
        one `message` object. Each `message` object is attributed to a specific entity
        through its `role`. The available roles are:

        - `user`: Represents the human querying the model. - `assistant`: Represents the
          model responding to user. - `system`: Represents a non-user entity that
          provides information to guide the behavior of the assistant.

        When the `role` of a `message` is set to `user`, `assistant`, or `system`, the
        `message` must also contain a `content` field which is a string representing the
        actual text of the message itself. Semantically, when the `role` is `user`,
        `content` contains the user's query. When the `role` is `assistant`, `content`
        is the model's response to the user. When the `role` is `system`, `content`
        represents the instruction for the assistant.

        ### Instructions

        You may provide instructions to the assistant by supplying by supplying
        `instructions` in the HTTP request body or by specifying a `message` with `role`
        set to `system` in the `messages` array. By convention, the system message
        should be the first message in the array. Do **not** specify both an instruction
        and a system message in the `messages` array.

        Args:
          messages: The list of messages in the conversation.

              Expand each message type to see how it works and when to use it. Most
              conversations should begin with a single `user` message.

          model: The ID of the model to use for chat completions. We only support the models
              listed here so far.

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          chat_template: Currently only supported for LLM-Engine models. A Jinja template string that
              defines how the chat completion API formats the string prompt. For Llama models,
              the template must take in at most a `messages` object, `bos_token` string, and
              `eos_token` string. The `messages` object is a list of dictionaries, each with
              keys `role` and `content`. For Mixtral models, the template must take in at most
              a `messages` object and `eos_token` string. The `messages` object looks
              identical to the Llama model's `messages` object, but the template can assume
              the `role` key takes on the values `user` or `assistant`, or `system` for the
              first message. The chat template either needs to handle this system message
              (which gets set via the `instructions` field or by the messages), or the
              `instructions` field must be set to `null` and the `messages` object must not
              contain any system messages.See the default chat template present in the Llama
              and Mixtral tokenizers for examples.

          instructions: The initial instructions to provide to the chat completion model.

              Use this to guide the model to act in more specific ways. For example, if you
              have specific rules you want to restrict the model to follow you can specify
              them here.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing more
              specific instructions here before trying more expensive techniques such as
              swapping in other models or finetuning the underlying LLM.

          memory_strategy: The memory strategy to use for the agent. A memory strategy is a way to prevent
              the underlying LLM's context limit from being exceeded. Each memory strategy
              uses a different technique to condense the input message list into a smaller
              payload for the underlying LLM.

              We only support the Last K memory strategy right now, but will be adding new
              strategies soon.

          model_parameters: Configuration parameters for the chat completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        messages: Iterable[chat_completion_create_params.Message],
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-4o",
            "gpt-4o-2024-08-06",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "gemini-pro",
            "gemini-1.5-pro-001",
            "gemini-1.5-pro-preview-0409",
            "gemini-1.5-pro-preview-0514",
            "llama-2-7b-chat",
            "llama-2-13b-chat",
            "llama-2-70b-chat",
            "llama-3-8b-instruct",
            "llama-3-70b-instruct",
            "llama-3-1-8b-instruct",
            "llama-3-1-70b-instruct",
            "Meta-Llama-3-8B-Instruct-RMU",
            "Meta-Llama-3-8B-Instruct-RR",
            "Meta-Llama-3-8B-Instruct-DERTA",
            "Meta-Llama-3-8B-Instruct-LAT",
            "mixtral-8x7b-instruct",
            "mixtral-8x22b-instruct",
            "claude-3-opus-20240229",
            "claude-3-sonnet-20240229",
            "claude-3-haiku-20240307",
            "claude-3-5-sonnet-20240620",
            "mistral-large-latest",
            "phi-3-mini-4k-instruct",
            "phi-3-cat-merged",
            "zephyr-cat-merged",
            "dolphin-2.9-llama3-8b",
            "dolphin-2.9-llama3-70b",
            "llama3-1-405b-instruct-v1",
        ],
        stream: bool,
        account_id: str | NotGiven = NOT_GIVEN,
        chat_template: str | NotGiven = NOT_GIVEN,
        instructions: str | NotGiven = NOT_GIVEN,
        memory_strategy: chat_completion_create_params.MemoryStrategy | NotGiven = NOT_GIVEN,
        model_parameters: chat_completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCompletionsResponse | AsyncStream[CompletionResponse]:
        """
        ### Description

        Given a list of messages representing a conversation history, runs LLM inference
        to produce the next message.

        ### Details

        Like [completions](https://scale-egp.readme.io/docs/completions-1),
        [chat completions](https://scale-egp.readme.io/docs/chat-completions-intro)
        involve an LLM's response to input. However, chat completions take a
        conversation history as input, instead of a single prompt, which enables the LLM
        to create responses that take past context into account.

        ### Messages

        The primary input to the LLM is a list of messages represented by the `messages`
        array, which forms the conversation. The `messages` array must contain at least
        one `message` object. Each `message` object is attributed to a specific entity
        through its `role`. The available roles are:

        - `user`: Represents the human querying the model. - `assistant`: Represents the
          model responding to user. - `system`: Represents a non-user entity that
          provides information to guide the behavior of the assistant.

        When the `role` of a `message` is set to `user`, `assistant`, or `system`, the
        `message` must also contain a `content` field which is a string representing the
        actual text of the message itself. Semantically, when the `role` is `user`,
        `content` contains the user's query. When the `role` is `assistant`, `content`
        is the model's response to the user. When the `role` is `system`, `content`
        represents the instruction for the assistant.

        ### Instructions

        You may provide instructions to the assistant by supplying by supplying
        `instructions` in the HTTP request body or by specifying a `message` with `role`
        set to `system` in the `messages` array. By convention, the system message
        should be the first message in the array. Do **not** specify both an instruction
        and a system message in the `messages` array.

        Args:
          messages: The list of messages in the conversation.

              Expand each message type to see how it works and when to use it. Most
              conversations should begin with a single `user` message.

          model: The ID of the model to use for chat completions. We only support the models
              listed here so far.

          stream: Whether or not to stream the response.

              Setting this to True will stream the completion in real-time.

          account_id: The account ID to use for usage tracking. This will be gradually enforced.

          chat_template: Currently only supported for LLM-Engine models. A Jinja template string that
              defines how the chat completion API formats the string prompt. For Llama models,
              the template must take in at most a `messages` object, `bos_token` string, and
              `eos_token` string. The `messages` object is a list of dictionaries, each with
              keys `role` and `content`. For Mixtral models, the template must take in at most
              a `messages` object and `eos_token` string. The `messages` object looks
              identical to the Llama model's `messages` object, but the template can assume
              the `role` key takes on the values `user` or `assistant`, or `system` for the
              first message. The chat template either needs to handle this system message
              (which gets set via the `instructions` field or by the messages), or the
              `instructions` field must be set to `null` and the `messages` object must not
              contain any system messages.See the default chat template present in the Llama
              and Mixtral tokenizers for examples.

          instructions: The initial instructions to provide to the chat completion model.

              Use this to guide the model to act in more specific ways. For example, if you
              have specific rules you want to restrict the model to follow you can specify
              them here.

              Good prompt engineering is crucial to getting performant results from the model.
              If you are having trouble getting the model to perform well, try writing more
              specific instructions here before trying more expensive techniques such as
              swapping in other models or finetuning the underlying LLM.

          memory_strategy: The memory strategy to use for the agent. A memory strategy is a way to prevent
              the underlying LLM's context limit from being exceeded. Each memory strategy
              uses a different technique to condense the input message list into a smaller
              payload for the underlying LLM.

              We only support the Last K memory strategy right now, but will be adding new
              strategies soon.

          model_parameters: Configuration parameters for the chat completion model, such as temperature,
              max_tokens, and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["messages", "model"], ["messages", "model", "stream"])
    async def create(
        self,
        *,
        messages: Iterable[chat_completion_create_params.Message],
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-4o",
            "gpt-4o-2024-08-06",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "gemini-pro",
            "gemini-1.5-pro-001",
            "gemini-1.5-pro-preview-0409",
            "gemini-1.5-pro-preview-0514",
            "llama-2-7b-chat",
            "llama-2-13b-chat",
            "llama-2-70b-chat",
            "llama-3-8b-instruct",
            "llama-3-70b-instruct",
            "llama-3-1-8b-instruct",
            "llama-3-1-70b-instruct",
            "Meta-Llama-3-8B-Instruct-RMU",
            "Meta-Llama-3-8B-Instruct-RR",
            "Meta-Llama-3-8B-Instruct-DERTA",
            "Meta-Llama-3-8B-Instruct-LAT",
            "mixtral-8x7b-instruct",
            "mixtral-8x22b-instruct",
            "claude-3-opus-20240229",
            "claude-3-sonnet-20240229",
            "claude-3-haiku-20240307",
            "claude-3-5-sonnet-20240620",
            "mistral-large-latest",
            "phi-3-mini-4k-instruct",
            "phi-3-cat-merged",
            "zephyr-cat-merged",
            "dolphin-2.9-llama3-8b",
            "dolphin-2.9-llama3-70b",
            "llama3-1-405b-instruct-v1",
        ],
        account_id: str | NotGiven = NOT_GIVEN,
        chat_template: str | NotGiven = NOT_GIVEN,
        instructions: str | NotGiven = NOT_GIVEN,
        memory_strategy: chat_completion_create_params.MemoryStrategy | NotGiven = NOT_GIVEN,
        model_parameters: chat_completion_create_params.ModelParameters | NotGiven = NOT_GIVEN,
        stream: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCompletionsResponse | AsyncStream[CompletionResponse]:
        return await self._post(
            "/v4/chat-completions",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "account_id": account_id,
                    "chat_template": chat_template,
                    "instructions": instructions,
                    "memory_strategy": memory_strategy,
                    "model_parameters": model_parameters,
                    "stream": stream,
                },
                chat_completion_create_params.ChatCompletionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCompletionsResponse,
            stream=stream or False,
            stream_cls=AsyncStream[CompletionResponse],
        )


class ChatCompletionsResourceWithRawResponse:
    def __init__(self, chat_completions: ChatCompletionsResource) -> None:
        self._chat_completions = chat_completions

        self.create = to_raw_response_wrapper(
            chat_completions.create,
        )


class AsyncChatCompletionsResourceWithRawResponse:
    def __init__(self, chat_completions: AsyncChatCompletionsResource) -> None:
        self._chat_completions = chat_completions

        self.create = async_to_raw_response_wrapper(
            chat_completions.create,
        )


class ChatCompletionsResourceWithStreamingResponse:
    def __init__(self, chat_completions: ChatCompletionsResource) -> None:
        self._chat_completions = chat_completions

        self.create = to_streamed_response_wrapper(
            chat_completions.create,
        )


class AsyncChatCompletionsResourceWithStreamingResponse:
    def __init__(self, chat_completions: AsyncChatCompletionsResource) -> None:
        self._chat_completions = chat_completions

        self.create = async_to_streamed_response_wrapper(
            chat_completions.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["EvaluationDatasetGenerationJob"]


class EvaluationDatasetGenerationJob(BaseModel):
    created_at: datetime
    """The timestamp at which the upload job started."""

    generation_job_id: str
    """ID of the async job associated with this evaluation dataset generation"""

    status: Literal["Pending", "Running", "Completed", "Failed", "Canceled"]
    """An enumeration."""

    updated_at: datetime
    """The timestamp at which the upload job was last updated."""

    failure_reason: Optional[str] = None
    """Reason for the job's failure, if applicable"""

    num_completed_test_cases: Optional[int] = None
    """Number of test cases that have been generated"""

    num_test_cases: Optional[int] = None
    """Optional number of test cases input to the job"""

    total_chunk_count: Optional[int] = None
    """Number of chunks in the knowledge base"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Any, List, Type, Generic, Mapping, TypeVar, Optional, cast
from typing_extensions import override

from httpx import Response

from ._utils import is_mapping
from ._models import BaseModel
from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = [
    "SyncPageResponse",
    "AsyncPageResponse",
    "SyncTopLevelArray",
    "AsyncTopLevelArray",
    "SyncChunkPagination",
    "AsyncChunkPagination",
    "SyncGenerationJobsPagination",
    "AsyncGenerationJobsPagination",
]

_BaseModelT = TypeVar("_BaseModelT", bound=BaseModel)

_T = TypeVar("_T")


class SyncPageResponse(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]
    current_page: Optional[int] = None

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = self.current_page
        if current_page is None:
            current_page = 1

        last_page = cast("int | None", self._options.params.get("page"))
        if last_page is not None and current_page <= last_page:
            # The API didn't return a new page in the last request
            return None

        return PageInfo(params={"page": current_page + 1})


class AsyncPageResponse(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]
    current_page: Optional[int] = None

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = self.current_page
        if current_page is None:
            current_page = 1

        last_page = cast("int | None", self._options.params.get("page"))
        if last_page is not None and current_page <= last_page:
            # The API didn't return a new page in the last request
            return None

        return PageInfo(params={"page": current_page + 1})


class SyncTopLevelArray(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> None:
        """
        This page represents a response that isn't actually paginated at the API level
        so there will never be a next page.
        """
        return None

    @classmethod
    def build(cls: Type[_BaseModelT], *, response: Response, data: object) -> _BaseModelT:  # noqa: ARG003
        return cls.construct(
            None,
            **{
                **(cast(Mapping[str, Any], data) if is_mapping(data) else {"items": data}),
            },
        )


class AsyncTopLevelArray(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> None:
        """
        This page represents a response that isn't actually paginated at the API level
        so there will never be a next page.
        """
        return None

    @classmethod
    def build(cls: Type[_BaseModelT], *, response: Response, data: object) -> _BaseModelT:  # noqa: ARG003
        return cls.construct(
            None,
            **{
                **(cast(Mapping[str, Any], data) if is_mapping(data) else {"items": data}),
            },
        )


class SyncChunkPagination(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    chunks: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        chunks = self.chunks
        if not chunks:
            return []
        return chunks

    @override
    def next_page_info(self) -> None:
        """
        This page represents a response that isn't actually paginated at the API level
        so there will never be a next page.
        """
        return None


class AsyncChunkPagination(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    chunks: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        chunks = self.chunks
        if not chunks:
            return []
        return chunks

    @override
    def next_page_info(self) -> None:
        """
        This page represents a response that isn't actually paginated at the API level
        so there will never be a next page.
        """
        return None


class SyncGenerationJobsPagination(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    generation_jobs: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        generation_jobs = self.generation_jobs
        if not generation_jobs:
            return []
        return generation_jobs

    @override
    def next_page_info(self) -> None:
        """
        This page represents a response that isn't actually paginated at the API level
        so there will never be a next page.
        """
        return None


class AsyncGenerationJobsPagination(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    generation_jobs: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        generation_jobs = self.generation_jobs
        if not generation_jobs:
            return []
        return generation_jobs

    @override
    def next_page_info(self) -> None:
        """
        This page represents a response that isn't actually paginated at the API level
        so there will never be a next page.
        """
        return None
